<?php

/**
 * Theme the table of Ooyala channels on the node edit form.
 *
 * @param $items
 *   An array of lineup items.
 *
 * @return
 *   The themed table.
 */
function theme_ooyala_channels_lineup_items_edit($items) {
  $rows = array();
  $output = "";
  $operations_heading = false;

  foreach(element_children($items) as $child) {
    $item = $items[$child];
    $row = array();
    $row[] = drupal_render($item['thumbnail']);
    $row[] = drupal_render($item['weight']);
    $row[] = drupal_render($item['title']);
    $row[] = drupal_render($item['status']);
    $row[] = drupal_render($item['length']);
    if (!empty($item['#operations'])) {
      $operations_heading = TRUE;
      // If this key has been set, the operations have been rendered for us.
      $row[] = $item['#operations'];
    }
    $rows[] = array(
      'data' => $row,
      'class' => 'draggable',
    );
  }

  // Only display the table if at least one row has been created.
  if (!empty($rows)) {
    $headings = array('', t('Weight'), t('Title'), t('Status'), t('Length'));
    if ($operations_heading) {
      $headings[] = t('Operations');
    }
    $output .= theme('table', $headings, $rows, array('id' => 'ooyala-channels-lineup-edit'));
  }

  return $output;
}

/**
 * Function to render the channel lineup. This function splits the radios
 * element up so it is rendered within each row's operations column.
 *
 * @param $element
 *   The form element to render.
 *
 * @return
 *   The themed HTML of the channel lineup.
 */
function theme_ooyala_channels_lineup($elements) {
  // This gives us radio elements from the radios element.
  if (!isset($elements['lineup'])) {
    return "";
  }

  $elements['thumbnail'] = expand_radios($elements['thumbnail']);

  foreach (element_children($elements['lineup']) as $child) {
    // We render the checkbox element (which is keyed by embed code) and then
    // append the Remove checkbox.
    $embed_code = $elements['lineup'][$child]['embedCode']['#value'];
    $elements['lineup'][$child]['#operations'] = drupal_render($elements['thumbnail'][$embed_code]);
    $elements['lineup'][$child]['#operations'] .= drupal_render($elements['lineup'][$child]['remove']);
  }

  // While we've rendered each radio element, we haven't rendered the radios
  // wrapper element. Set #printed to TRUE so drupal_render() skips the
  // element.
  $elements['thumbnail']['#printed'] = TRUE;

  // Note this will call theme_ooyala_channels_lineup_items_edit() for us.
  return drupal_render($elements);
}

/**
 * Helper theme function to add tabledrag for the channel lineup table. We use
 * a separate function so that it can be added to a form even if the table is
 * not rendered so tabledrag will work on all #ahah elements.
 */
function theme_ooyala_channels_lineup_tabledrag() {
  drupal_add_tabledrag('ooyala-channels-lineup-edit', 'order', 'self', 'ooyala-channels-lineup-edit-weight');
}

